@@ -164,7 +164,7 @@ sub post_search
         if ($rule->{context} eq 'searchResult') {
             my %rwcache;
             for (my $i = 0; $i < @{$entriesp}; $i++) {
-                my $entryStr = encode('utf8', ${$entriesp}[$i]);
+                my $entryStr = ${$entriesp}[$i];
                 my (@line) = split(/\n/, $entryStr);
                 my ($dn) = ($line[0] =~ /^dn: (.*)$/);
                 if (defined($rule->{dn}) && $dn !~ /$rule->{dn}/i) {
@@ -197,7 +197,7 @@ sub post_search
                     }
                 }
                 if (!$rc) {
-                    ${$entriesp}[$i] = decode('utf8', "$line[0]\n".join("\n", $self->_unique(@replaced))."\n");
+                    ${$entriesp}[$i] = "$line[0]\n".join("\n", $self->_unique(@replaced))."\n";
                 }
             }
         }
@@ -251,7 +251,9 @@ sub pre_modify
                     push(@values, shift @{$listp});
                 }
 
-                if (@values) {
+                if (defined($rule->{modop}) && $rule->{modop} !~ /$action/i) {
+                    push(@mod_list, ($action, $attr, @values));
+                } elsif (@values) {
                     for (my $i =0; $i < @values; $i++) {
                         $str = $self->_rewriteParse($rule->{match}, $substitution, "$attr: ".$values[$i], \%rwcache);
                         if (!defined($str)) {
@@ -471,7 +473,7 @@ sub _rewritePattern
         $value =~ s/$qt/\\$qt/g;
     }
 
-    $str =~ s/$pattern/$value/;
+    $str =~ s/$pattern/$value/g;
     return $str;
 }
 
@@ -499,7 +501,7 @@ sub _rewriteParse
 
         # do functions
         my @substs = ($substline);
-        my @rwmaps = ($substline =~ /%{([^(]*\(.*\))}/gs);
+        my @rwmaps = ($substline =~ /%{([^(]*\((?:(?!%{).)*\))}/gs);
         foreach my $rwmap (@rwmaps) {
             my @values;
             my $key = lc($rwmap);
