@@ -114,6 +114,17 @@ sub _checkConfig
         $conf->{valdelim}[0] = ';';
     }
 
+    foreach my $oname (keys %{$conf->{object}}) {
+        my $oconf = $conf->{object}{$oname};
+
+        foreach my $attr (keys %{$oconf->{attr}}) {
+            if (defined($oconf->{attr}{$attr}->{rexpr})) {
+                ($oconf->{attr}{$attr}->{rexpr_expr} = $oconf->{attr}{$attr}->{rexpr}[0]) =~ s/\%[0-9]+/(.+)/g;
+                $oconf->{attr}{$attr}->{rexpr_columns} = [($oconf->{attr}{$attr}->{rexpr}[0] =~ /\%([0-9]+)/g)];
+            }
+        }
+    }
+
     return $rc;
 }
 
@@ -146,11 +157,18 @@ sub _objSearch
         }
 
         while (<$file>) {
-            s/\r|\n|\r\n$//;
+            s/\r\n$//;
+            s/\r$//;
+            s/\n$//;
             my @data = split(/$dlm/);
             my $entry;
             my $match = 1;
 
+            for (my $i = 0; $i < @data; $i++) {
+                $data[$i] =~ s/^["' ]//;
+                $data[$i] =~ s/["' ]$//;
+            }
+
             foreach my $strginfo (@{$oconf->{strginfo}}) {
                 if (defined($strginfo->{column}) && defined($strginfo->{filter}) && defined($strginfo->{filter}->{type})) {
                     my $value = $data[$strginfo->{column}[0]];
@@ -199,9 +217,18 @@ sub _objSearch
             }
             foreach my $attr (keys %{$oconf->{attr}}) {
                 if (defined($oconf->{attr}{$attr}->{column})) {
-                    foreach my $val (split(/$valdlm/, $data[$oconf->{attr}{$attr}->{column}[0]])) {
-                        $val =~ s/ *$//;
-                        $entry = $entry."$attr: $val\n";
+                    foreach my $value (split(/$valdlm/, $data[$oconf->{attr}{$attr}->{column}[0]])) {
+                        $value =~ s/ *$//;
+                        $entry = $entry."$attr: $value\n";
+                    }
+                } elsif (defined($oconf->{attr}{$attr}->{rexpr})) {
+                    my $value = $oconf->{attr}{$attr}->{rexpr}[0];
+                    for (my $i = @data; $i >= 0; $i--) {
+                        $value =~ s/%$i/$data[$i]/;
+                    }
+                    $value =~ s/ *$//;
+                    if ($value) {
+                        $entry = $entry."$attr: $value\n";
                     }
                 } else {
                     my $values = $self->_getAttrValues($oconf, $attr, split(/$valdlm/, $data[$oconf->{attr}{$attr}->{idcolumn}[0]]));
@@ -219,6 +246,12 @@ sub _objSearch
             $entry = decode($conf->{mbcode}[0], $entry);
 
             my ($rdn_val) = ($entry =~ /^$oconf->{rdn}[0]: (.*)$/mi);
+            if (!$rdn_val) {
+                $self->log(level => 'err', message => "Entry doesn't have RDN");
+                $rc = LDAP_OTHER;
+                last;
+            }
+
             my $rdn = "$oconf->{rdn}[0]=$rdn_val";
             $entry = "dn: $rdn,$suffix\n$entry";
 
@@ -472,49 +505,57 @@ sub _objAdd
     }
     flock($lock, 2);
 
-    # check whether the entry already exists
-    if (!rename($oconf->{file}[0], "$oconf->{file}[0].tmp")) {
-        $self->log(level => 'alert', message => "Can't rename $oconf->{file}[0] to $oconf->{file}[0].tmp");
-        close($lock);
-        return LDAP_OPERATIONS_ERROR;
-    }
-
-    if (!open($tmp, "< $oconf->{file}[0].tmp")) {
-        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].tmp");
-        close($lock);
-        return LDAP_OPERATIONS_ERROR;
-    }
-
-    if (!open($file, "> $oconf->{file}[0]")) {
-        $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
-        close($tmp);
-        close($lock);
-        return LDAP_OPERATIONS_ERROR;
-    }
-
-    while (<$tmp>) {
-        print $file $_;
-
-        chop;
-        my @data = split(/$dlm/);
-
-        # check the data correspods to the dn
-        if ("$valdlm$data[$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]]$valdlm" !~ /$valdlm *$rdn_val *$valdlm/i) {
-            next;
+    if (defined($oconf->{mode}[0]) && $oconf->{mode}[0] eq 'append') {
+        if (!open($file, ">> $oconf->{file}[0]")) {
+            $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
+            close($lock);
+            return LDAP_OPERATIONS_ERROR;
+        }
+    } else {
+        # check whether the entry already exists
+        if (!rename($oconf->{file}[0], "$oconf->{file}[0].tmp")) {
+            $self->log(level => 'alert', message => "Can't rename $oconf->{file}[0] to $oconf->{file}[0].tmp");
+            close($lock);
+            return LDAP_OPERATIONS_ERROR;
         }
 
-        # entries below suffix
-        if (defined($pkey) && defined($oconf->{container}) && !defined($oconf->{container}[0]->{rdn})) {
-            my $cur_pkey = $data[$oconf->{container}[0]->{idcolumn}[0]];
-            if ($pkey !~ /^$cur_pkey$/i) {
+        if (!open($tmp, "< $oconf->{file}[0].tmp")) {
+            $self->log(level => 'alert', message => "Can't open $oconf->{file}[0].tmp");
+            close($lock);
+            return LDAP_OPERATIONS_ERROR;
+        }
+
+        if (!open($file, "> $oconf->{file}[0]")) {
+            $self->log(level => 'alert', message => "Can't open $oconf->{file}[0]");
+            close($tmp);
+            close($lock);
+            return LDAP_OPERATIONS_ERROR;
+        }
+
+        while (<$tmp>) {
+            print $file $_;
+
+            chop;
+            my @data = split(/$dlm/);
+
+            # check the data correspods to the dn
+            if ("$valdlm$data[$oconf->{attr}{$oconf->{rdn}[0]}->{column}[0]]$valdlm" !~ /$valdlm *$rdn_val *$valdlm/i) {
                 next;
             }
-        }
 
-        $rc = LDAP_ALREADY_EXISTS;
-        last;
+            # entries below suffix
+            if (defined($pkey) && defined($oconf->{container}) && !defined($oconf->{container}[0]->{rdn})) {
+                my $cur_pkey = $data[$oconf->{container}[0]->{idcolumn}[0]];
+                if ($pkey !~ /^$cur_pkey$/i) {
+                    next;
+                }
+            }
+
+            $rc = LDAP_ALREADY_EXISTS;
+            last;
+        }
+        close($tmp);
     }
-    close($tmp);
 
     if (!$rc) {
         my @data;
@@ -529,24 +570,39 @@ sub _objAdd
                     last DO;
                 }
 
-                if (defined($oconf->{attr}{$attr}->{column})) {
-                    $coln = $oconf->{attr}{$attr}->{column}[0];
-                } else {
-                    my @keys;
-                    $coln = $oconf->{attr}{$attr}->{idcolumn}[0];
-
-                    # convert the value to object's id
-                    ($rc, @keys) = $self->_getAttrKeys($oconf, $attr, @values);
-                    if ($rc) {
-                        $self->log(level => 'err', message => "Can't get id of $attr values in the file");
-                        $rc = LDAP_OTHER;
-                        last DO;
+                if (defined($oconf->{attr}{$attr}->{rexpr})) {
+                    foreach my $value (@values) {
+                        my @colvals = ($value =~ $oconf->{attr}{$attr}->{rexpr_expr});
+                        for (my $i = 0; $i < @colvals; $i++) {
+                            my $colval = $data[$oconf->{attr}{$attr}->{rexpr_columns}[$i]];
+                            if (!$colval) {
+                                $colval = $colvals[$i];
+                            } elsif ("$valdlm$colval$valdlm" !~ /$valdlm$colvals[$i]$valdlm/i) {
+                                $colval = $colval.$valdlm.$colvals[$i];
+                            }
+                            $data[$oconf->{attr}{$attr}->{rexpr_columns}[$i]] = $colval;
+                        }
                     }
-                    @values = @keys;
-                }
+                } else {
+                    if (defined($oconf->{attr}{$attr}->{column})) {
+                        $coln = $oconf->{attr}{$attr}->{column}[0];
+                    } else {
+                        my @keys;
+                        $coln = $oconf->{attr}{$attr}->{idcolumn}[0];
 
-                if ($data[$coln] =~ /^ *$/ || @values > split(/$valdlm/, $data[$coln])) {
-                    $data[$coln] = join($valdlm, @values);
+                        # convert the value to object's id
+                        ($rc, @keys) = $self->_getAttrKeys($oconf, $attr, @values);
+                        if ($rc) {
+                            $self->log(level => 'err', message => "Can't get id of $attr values in the file");
+                            $rc = LDAP_OTHER;
+                            last DO;
+                        }
+                        @values = @keys;
+                    }
+
+                    if ($data[$coln] =~ /^ *$/ || @values > split(/$valdlm/, $data[$coln])) {
+                        $data[$coln] = join($valdlm, @values);
+                    }
                 }
             }
 
