@@ -161,6 +161,8 @@ sub bind
             if ($binddn =~ /^$ldapmap->{local}=/i && (!defined($ldapmap->{dn}) || $binddn =~ /$ldapmap->{dn}/i)) {
                 $binddn = $self->_rewriteDn($ldapmap, 'request', $binddn);
             }
+        } elsif ($ldapmap->{type} =~ /^attribute$/i) {
+            $binddn =~ s/^$ldapmap->{local}=/$ldapmap->{foreign}=/i;
         }
     }
     $binddn =~ s/$self->{suffix}$/$conf->{nc}/i;
@@ -218,6 +220,7 @@ sub compare
                 $avaStr =~ s/^$ldapmap->{local}$/$ldapmap->{foreign}/i;
             }
         } elsif ($ldapmap->{type} =~ /^attribute$/i) {
+            $dn =~ s/^$ldapmap->{local}=/$ldapmap->{foreign}=/i;
             if ($key =~ /^$ldapmap->{local}$/i) {
                 $key = $ldapmap->{foreign};
             }
@@ -247,6 +250,7 @@ sub modify
     my ($dn, @list) = @_;
     my $conf = $self->{_config};
     my $rc = LDAP_SUCCESS;
+    my $foreign_rdnattr;
 
     if (defined($conf->{noop}) && grep(/^modify$/i, @{$conf->{noop}})) {
         return $rc;
@@ -257,12 +261,19 @@ sub modify
         return $rc;
     }
 
+    if ($self->_getConnect()) {
+        return LDAP_SERVER_DOWN;
+    }
+
     # DN mapping
     foreach my $ldapmap (@{$conf->{ldapmap}}) {
         if ($ldapmap->{type} =~ /^dn$/i) {
             if ($dn =~ /^$ldapmap->{local}=/i && (!defined($ldapmap->{dn}) || $dn =~ /$ldapmap->{dn}/i)) {
                 $dn = $self->_rewriteDn($ldapmap, 'request', $dn);
+                $foreign_rdnattr = lc($ldapmap->{foreign});
             }
+        } elsif ($ldapmap->{type} =~ /^attribute$/i) {
+            $dn =~ s/^$ldapmap->{local}=/$ldapmap->{foreign}=/i;
         }
     }
     $dn =~ s/$self->{suffix}$/$conf->{nc}/i;
@@ -281,6 +292,10 @@ sub modify
             next;
         }
 
+        if ($foreign_rdnattr && $key eq $foreign_rdnattr) {
+            next;
+        }
+
         if ($key eq 'entrycsn') {
             last;
         }
@@ -320,10 +335,6 @@ sub modify
         }
     }
 
-    if ($self->_getConnect()) {
-        return LDAP_SERVER_DOWN;
-    }
-
     my $msg = $self->{ldap}->modify($dn, changes => [@changes]);
 
     $self->_freeConnect($msg);
@@ -368,6 +379,8 @@ sub add
                     $dn =~ s/^[^,]+/$ldapmap->{foreign}=$rdn_val/i;
                 }
             }
+        } elsif ($ldapmap->{type} =~ /^attribute$/i) {
+            $dn =~ s/^$ldapmap->{local}=/$ldapmap->{foreign}=/i;
         }
     }
     $dn =~ s/$self->{suffix}$/$conf->{nc}/i;
@@ -437,6 +450,8 @@ sub modrdn
             if ($dn =~ /^$ldapmap->{local}=/i && (!defined($ldapmap->{dn}) || $dn =~ /$ldapmap->{dn}/i)) {
                 $dn = $self->_rewriteDn($ldapmap, 'request', $dn);
             }
+        } elsif ($ldapmap->{type} =~ /^attribute$/i) {
+            $dn =~ s/^$ldapmap->{local}=/$ldapmap->{foreign}=/i;
         }
     }
 
@@ -468,26 +483,48 @@ sub delete
         return $rc;
     }
 
+    if (defined($conf->{deleteflag})) {
+        my $entryStr;
+        ($rc, $entryStr) = $self->_do_search(undef, $dn, 0, 0, 1, 0, '(objectClass=*)');
+        if ($rc) {
+            return $rc;
+        }
+
+        my $match = 0;
+        foreach my $key (keys %{$conf->{deleteflag}}) {
+            my $deleteflag = $conf->{deleteflag}{$key};
+            if (!defined($deleteflag->{filter}) || $self->parseFilter($deleteflag->{filterobj}, $entryStr)) {
+                $rc = $self->modify($dn, "REPLACE", $key, $deleteflag->{value});
+                $match = 1;
+            }
+        }
+        if ($match) {
+            return $rc;
+        }
+    }
+
     $rc = $self->_beginWork('delete', $dn);
     if ($rc) {
         return $rc;
     }
 
+    if ($self->_getConnect()) {
+        return LDAP_SERVER_DOWN;
+    }
+
     # DN mapping
     foreach my $ldapmap (@{$conf->{ldapmap}}) {
         if ($ldapmap->{type} =~ /^dn$/i) {
             if ($dn =~ /^$ldapmap->{local}=/i && (!defined($ldapmap->{dn}) || $dn =~ /$ldapmap->{dn}/i)) {
                 $dn = $self->_rewriteDn($ldapmap, 'request', $dn);
             }
+        } elsif ($ldapmap->{type} =~ /^attribute$/i) {
+            $dn =~ s/^$ldapmap->{local}=/$ldapmap->{foreign}=/i;
         }
     }
 
     $dn =~ s/$self->{suffix}$/$conf->{nc}/i;
 
-    if ($self->_getConnect()) {
-        return LDAP_SERVER_DOWN;
-    }
-
     my $msg = $self->{ldap}->delete($dn);
 
     $self->_freeConnect($msg);
@@ -535,7 +572,8 @@ sub _getConnect
     my $self = shift;
     my $conf = $self->{_config};
 
-    if (defined($self->{ldap}) && defined($self->{bind})) {
+    if (defined($self->{ldap}) && defined($self->{bind}) &&
+        (!defined($conf->{connection}[0]->{type}) || $conf->{connection}[0]->{type}[0] ne 'every')) {
         my $msg = $self->{ldap}->bind($conf->{binddn}[0], password => $conf->{bindpw}[0]);
         if ($msg->code) {
             $self->log(level => 'err', message => "Connection check failed");
@@ -568,13 +606,15 @@ sub _getConnect
 sub _freeConnect
 {
     my $self =shift;
+    my $conf = $self->{_config};
     my ($msg) = @_;
 
     if (!$msg) {
         return -1;
     }
 
-    if ($msg->code == LDAP_SERVER_DOWN || $msg->code == -1) {
+    if ($msg->code == LDAP_SERVER_DOWN || $msg->code == -1 ||
+        (defined($conf->{connection}[0]->{type}) && $conf->{connection}[0]->{type}[0] eq 'every')) {
         $self->{ldap}->unbind();
         $self->{bind}->unbind();
 
@@ -599,6 +639,14 @@ sub _checkConfig
         $conf->{transaction}[0] = 'off';
     }
 
+    if (defined($conf->{deleteflag})) {
+        foreach my $key (keys %{$conf->{deleteflag}}) {
+            if ($conf->{deleteflag}{$key}->{filter}) {
+                $conf->{deleteflag}{$key}->{filterobj} = Net::LDAP::Filter->new($conf->{deleteflag}{$key}->{filter});
+            }
+        }
+    }
+
     foreach my $ldapmap (@{$conf->{ldapmap}}) {
         if (!defined($ldapmap->{type})) {
             $ldapmap->{type} = 'attribute';
@@ -677,7 +725,8 @@ sub _do_search
         } elsif ($ldapmap->{type} =~ /^objectclass$/i) {
             $filterStr =~ s/objectClass=$ldapmap->{local}/objectClass=$ldapmap->{foreign}/mi;
         } elsif ($ldapmap->{type} =~ /^attribute$/i) {
-            $filterStr =~ s/\($ldapmap->{local}=/\($ldapmap->{foreign}=/mi;
+            $base =~ s/^$ldapmap->{local}=/$ldapmap->{foreign}=/i;
+            $filterStr =~ s/\($ldapmap->{local}=/\($ldapmap->{foreign}=/gmi;
         }
     }
 
@@ -735,7 +784,8 @@ sub _do_search
                 if ($ldapmap->{type} =~ /^objectclass$/i) {
                     $entryStr =~ s/^objectClass: $ldapmap->{foreign}$/objectClass: $ldapmap->{local}/mi;
                 } elsif ($ldapmap->{type} =~ /^attribute$/i) {
-                    $entryStr =~ s/^$ldapmap->{foreign}:/$ldapmap->{local}:/mi;
+                    $entryStr =~ s/^$ldapmap->{foreign}:/$ldapmap->{local}:/gmi;
+                    $entryStr =~ s/: $ldapmap->{foreign}=/: $ldapmap->{local}=/gi;
                 }
             }
 
