@@ -139,7 +139,7 @@ sub _connect
     my ($db) = @_;
     my $conf = $self->{_config};
 
-    if (defined(${$db})) {
+    if (defined(${$db}) && (!defined($conf->{connection}[0]->{type}) || $conf->{connection}[0]->{type}[0] ne 'every')) {
         my $sth = ${$db}->prepare("select 'check';");
         if (!$sth->execute) {
             $self->log(level => 'err', message => "Connection check failed");
@@ -168,8 +168,9 @@ sub _connect
 sub _freeConnect
 {
     my $self = shift;
+    my $conf = $self->{_config};
 
-    if ($self->{db}->err) {
+    if ($self->{db}->err || (defined($conf->{connection}[0]->{type}) && $conf->{connection}[0]->{type}[0] eq 'every')) {
         $self->{db}->disconnect();
 
         undef($self->{db});
@@ -231,7 +232,7 @@ Search the appropriate records in the object's table.
 sub _objSearch
 {
     my $self = shift;
-    my ($obj, $pkeys, $suffix, $sizeLim, $filter) = @_;
+    my ($obj, $pkeys, $suffix, $sizeLim, $filter, $attrOnly, @attrs) = @_;
     my $conf = $self->{_config};
     my $oconf = $obj->{conf};
     my $pkey = ${$pkeys}[$#{$pkeys}];
@@ -314,25 +315,53 @@ sub _objSearch
 
             my $j = 0;
             foreach my $attr ('objectclass', keys %{$oconf->{attr}}) {
+                if ($attr !~ /^objectclass$/i && !defined($oconf->{attr}->{$attr})) {
+                    next;
+                }
+
+                if (@attrs && $attr !~ /^(objectclass|$oconf->{rdn}[0])$/i && !grep(/^$attr$/i, @attrs)) {
+                    if (defined($oconf->{attr}{$attr}->{column})) {
+                        $j++;
+                    }
+                    next;
+                }
+
                 if ($attr =~ /^objectclass$/i) {
                     foreach my $oc (@{$oconf->{oc}}) {
                         $entry = $entry."objectclass: $oc\n";
                     }
-                } elsif (!defined($oconf->{attr}->{$attr})) {
-                    next;
                 } elsif (defined($oconf->{attr}{$attr}->{column})) {
-                    my $value = $data[$j];
-                    $value =~ s/ *$//;
-                    if ($value) {
-                        if ($attr =~ /$rawattrs/i) {
-                            $value = encode_base64($value, '');
-                            $entry = $entry.$attr.":: $value\n";
-                        } elsif ($value =~ /\n/) {
-                            Encode::from_to($value, $conf->{mbcode}[0], 'utf8');
-                            $value = encode_base64($value, '');
-                            $entry = $entry.$attr.":: $value\n";
-                        } else {
-                            $entry = $entry."$attr: $value\n";
+                    if (defined($oconf->{attr}{$attr}->{delim})) {
+                        my @values = split($oconf->{attr}{$attr}->{delim}[0], $data[$j]);
+                        foreach my $value (@values) {
+                            $value =~ s/ *$//;
+                            if ($value) {
+                                if ($attr =~ /$rawattrs/i) {
+                                    $value = encode_base64($value, '');
+                                    $entry = $entry.$attr.":: $value\n";
+                                } elsif ($value =~ /\n/) {
+                                    Encode::from_to($value, $conf->{mbcode}[0], 'utf8');
+                                    $value = encode_base64($value, '');
+                                    $entry = $entry.$attr.":: $value\n";
+                                } else {
+                                    $entry = $entry."$attr: $value\n";
+                                }
+                            }
+                        }
+                    } else {
+                        my $value = $data[$j];
+                        $value =~ s/ *$//;
+                        if ($value) {
+                            if ($attr =~ /$rawattrs/i) {
+                                $value = encode_base64($value, '');
+                                $entry = $entry.$attr.":: $value\n";
+                            } elsif ($value =~ /\n/) {
+                                Encode::from_to($value, $conf->{mbcode}[0], 'utf8');
+                                $value = encode_base64($value, '');
+                                $entry = $entry.$attr.":: $value\n";
+                            } else {
+                                $entry = $entry."$attr: $value\n";
+                            }
                         }
                     }
                     $j++;
@@ -474,12 +503,19 @@ sub _objModify
                 }
 
                 if (defined($oconf->{attr}{$attr}->{column})) {
-                    # the attribute must not exist
-                    if ($entry =~ /^$attr: /mi) {
+                    my $value = $values[0];
+                    if (defined($oconf->{attr}{$attr}->{delim})) {
+                        my @old_vals = ($entry =~ /^$attr: (.*)$/gmi);
+                        $value = join($oconf->{attr}{$attr}->{delim}[0], (@old_vals, @values));
+                        foreach my $new_val (@old_vals, @values) {
+                            $entry = "$entry$attr: $new_val\n";
+                        }
+                    } elsif ($entry =~ /^$attr: /mi) {
+                        # the attribute must not exist
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                         last DO;
                     }
-                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$values[0]\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$value\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], $sql);
@@ -511,7 +547,20 @@ sub _objModify
                 }
 
                 if (defined($oconf->{attr}{$attr}->{column})) {
-                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
+                    my $value = '';
+                    if (defined($oconf->{attr}{$attr}->{delim}) && $values[0] ne '') {
+                        my @old_vals = ($entry =~ /^$attr: (.*)$/gmi);
+                        my @new_vals = ();
+                        for (my $i = 0; $i < @old_vals; $i++) {
+                            if (!grep(/^$old_vals[$i]$/i, @values)) {
+                                push(@new_vals, $old_vals[$i]);
+                            }
+                        }
+                        if (@new_vals) {
+                            $value = join($oconf->{attr}{$attr}->{delim}[0], @new_vals);
+                        }
+                    }
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$value\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], $sql);
@@ -542,12 +591,15 @@ sub _objModify
                 }
             } elsif( $action eq "REPLACE" ) {
                 if (defined($oconf->{attr}{$attr}->{column})) {
-                    # the attribute must not have more than two value
-                    if (@values > 1) {
+                    my $value = $values[0];
+                    if (defined($oconf->{attr}{$attr}->{delim})) {
+                        $value = join($oconf->{attr}{$attr}->{delim}[0], @values);
+                    } elsif (@values > 1) {
+                        # the attribute must not have more than two value
                         $rc = LDAP_CONSTRAINT_VIOLATION;
                         last DO;
                     }
-                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$values[0]\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
+                    $sql = "update $oconf->{table}[0] set $oconf->{attr}{$attr}->{column}[0] = \'$value\' where $oconf->{id}[0]->{column}[0] = \'$key\'";
 
                     # multibyte value
                     $sql = encode($conf->{mbcode}[0], $sql);
@@ -679,7 +731,20 @@ sub _objAdd
             $value =~ s/\\/\\\\/g;
 
             if (defined($oconf->{attr}{$attr}->{column})) {
-                if (!grep(/^$oconf->{attr}{$attr}->{column}[0]$/, @cols)) {
+                if (defined($oconf->{attr}{$attr}->{delim})) {
+                    my $match = 0;
+                    for (my $i = 0; $i < @cols; $i++) {
+                        if ($cols[$i] =~ /^$oconf->{attr}{$attr}->{column}[0]$/) {
+                            $values[$i] =~ s/^'(.+)'$/\'$1$oconf->{attr}{$attr}->{delim}[0]$value\'/;
+                            $match = 1;
+                            last;
+                        }
+                    }
+                    if (!$match) {
+                        push(@cols, $oconf->{attr}{$attr}->{column}[0]);
+                        push(@values, "\'$value\'");
+                    }
+                } elsif (!grep(/^$oconf->{attr}{$attr}->{column}[0]$/, @cols)) {
                     push(@cols, $oconf->{attr}{$attr}->{column}[0]);
                     push(@values, "\'$value\'");
                 }
@@ -1081,7 +1146,7 @@ sub _addAttrValues
     my $conf = $self->{_config};
 
     if (!defined($aconf->{addproc})) {
-        return -1;
+        return 0;
     }
 
     foreach my $value (@values) {
@@ -1155,7 +1220,7 @@ sub _delAttrValues
     my $conf = $self->{_config};
 
     if (!defined($aconf->{delproc})) {
-        return 1;
+        return 0;
     }
 
     foreach my $value (@values) {
@@ -1282,10 +1347,14 @@ sub _filter2sql
         $type = 'like';
         if ($op eq 'substrings') {
             $attr = lc($args->{type});
-            $value = $args->{substrings}[0]{initial}."%".$args->{substrings}[1]{final};
+            if (defined($args->{substrings}[0]{any})) {
+                $value = "%".$args->{substrings}[0]{any}."%";
+            } else {
+                $value = $args->{substrings}[0]{initial}."%".$args->{substrings}[1]{final};
+            }
         } elsif ($op eq 'present') {
             $attr = lc($args);
-            $value = '%';
+            $value = '_%';
         }
 
         if (defined($oconf->{attr}{$attr}) && defined($oconf->{attr}{$attr}->{oname})) {
@@ -1303,6 +1372,7 @@ sub _filter2sql
     ${$from} = $oconf->{table}[0];
     if ($attr =~ /^objectclass$/i) {
         # check object class in filter
+        $value =~ s/_/./;
         $value =~ s/%/.*/;
         if (grep(/^$value$/i, @{$oconf->{oc}})) {
             ${$where} = '1 = 1';
