@@ -1,4 +1,10 @@
 #!/usr/bin/perl
+
+eval 'exec /usr/bin/perl  -S $0 ${1+"$@"}'
+    if 0; # not running under some shell
+
+eval 'exec /usr/bin/perl  -S $0 ${1+"$@"}'
+    if 0; # not running under some shell
 #
 #  This code was developped by SECIOSS (http://www.secioss.co.jp/).
 #
@@ -12,6 +18,7 @@ use strict;
 use Time::Local;
 use Time::HiRes qw(gettimeofday);
 use POSIX;
+use Encode;
 use Data::Dumper;
 
 sub date2time
@@ -42,9 +49,38 @@ sub time2date
 
 sub getValue
 {
-    my ($entryStr, $attr) = @_;
+    my ($entryStr, $attr, $default) = @_;
 
-    return ($entryStr =~ /^$attr:\:? (.*)$/mi)[0];
+    my $value = ($entryStr =~ /^$attr:\:? (.*)$/mi)[0];
+    if (!defined($value) && $default) {
+        $value = $default;
+    }
+
+    return $value;
+}
+
+sub getSureName
+{
+    my ($entryStr, $attr) = @_;
+    my $regexp = "^$attr: ([^ 　,，\n]+)";
+    $regexp = decode('utf8', $regexp);
+    my $value = ($entryStr =~ /$regexp/mi)[0];
+
+    return $value;
+}
+
+sub getGivenName
+{
+    my ($entryStr, $attr, $default) = @_;
+    my $regexp = "^$attr: [^ 　,，\n]+[ 　,，]+([^\n]+)".'$';
+    $regexp = decode('utf8', $regexp);
+    my $value = ($entryStr =~ /$regexp/mi)[0];
+
+    if ($value) {
+        return $value;
+    } else {
+        return $default;
+    }
 }
 
 sub getFileContents
